package com.tunyk.jsonbatchtranslate;

import com.google.api.translate.Language;
import com.google.api.translate.Translate;
import com.tunyk.jsonbatchtranslate.api.JsonBatchTranslate;
import com.tunyk.jsonbatchtranslate.api.JsonBatchTranslateException;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.io.InputStream;
import java.util.*;

public class JsonBatchTranslateImpl implements JsonBatchTranslate {

    private static final String PROPERTIES_FILENAME = "jsonbatchtranslate.properties";
    private static final String PROPERTY_HTTP_REFERRER = "jsonbatchtranslate.httpReferrer";

    private Properties properties;
    private JSONObject jsonRef;
    private Map<Long, String> stringMap;
    private static Long uid;
    private static Long startUID;

    public JsonBatchTranslateImpl() throws JsonBatchTranslateException {
        InputStream inputStream  = JsonBatchTranslateImpl.class.getClassLoader().getResourceAsStream(PROPERTIES_FILENAME);
        this.properties = new Properties();
        try {
            this.properties.load(inputStream);
        } catch (IOException e) {
            throw new JsonBatchTranslateException(e);
        }
        try {
            inputStream.close();
        } catch (IOException e) {
            throw new JsonBatchTranslateException(e);
        }
        Translate.setHttpReferrer(this.properties.getProperty(PROPERTY_HTTP_REFERRER));
    }

    public JSONObject translate(JSONObject json, com.tunyk.jsonbatchtranslate.api.Language fromLang, com.tunyk.jsonbatchtranslate.api.Language toLang) throws JsonBatchTranslateException, JSONException {
        if (json == null) {
            return null;
        }

        String[] textToTranslate = convertJsonToStringArray(json);
        String[] translatedText;

        try {
            translatedText = Translate.execute(textToTranslate,
                     Language.fromString(fromLang.toString()), Language.fromString(toLang.toString()));
        } catch (Exception e) {
            throw new JsonBatchTranslateException(e);
        }

        return integrateTranslationToJson(translatedText);
    }

    /*public JSONObject translate(JSONObject json, com.tunyk.jsonbatchtranslate.api.Language fromLang, com.tunyk.jsonbatchtranslate.api.Language toLang, List<String> propertiesToTranslate) throws JsonBatchTranslateException, JSONException {
        if (json == null) {
            return null;
        }

        String[] textToTranslate = convertJsonToStringArray(json, propertiesToTranslate);
        String[] translatedText;

        try {
            translatedText = Translate.execute(textToTranslate,
                     Language.fromString(fromLang.toString()), Language.fromString(toLang.toString()));
        } catch (Exception e) {
            throw new JsonBatchTranslateException(e);
        }

        return integrateTranslationToJson(translatedText);
    }*/

    private String[] convertJsonToStringArray(JSONObject json) throws JSONException {
        uid = System.currentTimeMillis();
        startUID = uid;
        stringMap = new HashMap<Long, String>();
        jsonRef = iterateJSONObject(json);
        return convertStringMapToArray(stringMap);
    }

    private String[] convertJsonToStringArray(JSONObject json, List<String> propertiesToTranslate) throws JSONException {
        uid = System.currentTimeMillis();
        startUID = uid;
        stringMap = new HashMap<Long, String>();
        // TODO: after iterateJSONObject will be implemented completely provide propertiesToTranslate as param
        jsonRef = iterateJSONObject(json);
        return convertStringMapToArray(stringMap);
    }

    private JSONObject integrateTranslationToJson(String[] translatedText) throws JSONException {
        for (int i = 0; i < translatedText.length; i++) {
            stringMap.put(Long.valueOf(startUID + i), translatedText[i]);
        }

        return reverseIterateJSONObject(jsonRef);
    }

    private JSONObject iterateJSONObject(JSONObject json) throws JSONException {
        JSONObject res = new JSONObject();
        Iterator itr = json.keys();
        while (itr.hasNext()) {
            String key = (String) itr.next();
            Object obj = json.get(key);
            if (obj instanceof JSONObject) {
                JSONObject innerJson = iterateJSONObject((JSONObject) obj);
                res.put(key, innerJson);
            } else if (obj instanceof JSONArray) {
                JSONArray innerJson = iterateJSONArray((JSONArray) obj);
                res.put(key, innerJson);
            } else {
                // TODO: iterate through json and make hierarchical key, if it is provided for translation then include to stringMap. otherwise place original value.
                if (obj instanceof String) {
                    Long id = getUID();
                    stringMap.put(id, obj.toString());
                    res.put(key, id);
                } else {
                    res.put(key, obj);
                }
            }
        }
        return res;
    }

    private JSONArray iterateJSONArray(JSONArray json) throws JSONException {
        JSONArray res = new JSONArray();
        for (int i = 0; i < json.length(); i++) {
            Object obj = json.get(i);
            if (obj instanceof JSONObject) {
                JSONObject innerJson = iterateJSONObject((JSONObject) obj);
                res.put(i, innerJson);
            } else if (obj instanceof JSONArray) {
                JSONArray innerJson = iterateJSONArray((JSONArray) obj);
                res.put(i, innerJson);
            } else {
                // TODO: iterate through json and make hierarchical key, if it is provided for translation then include to stringMap. otherwise place original value.
                if (obj instanceof String) {
                    Long id = getUID();
                    stringMap.put(id, obj.toString());
                    res.put(i, id);
                } else {
                    res.put(i, obj);
                }
            }
        }
        return res;
    }

    private String[] convertStringMapToArray(Map<Long, String> map) {
        String[] res = new String[map.size()];
        for (int i = 0; i < map.size(); i++) {
            res[i] = map.get(Long.valueOf(startUID + i));
        }
        return res;
    }

    private JSONObject reverseIterateJSONObject(JSONObject json) throws JSONException {
        JSONObject res = new JSONObject();
        Iterator itr = json.keys();
        while (itr.hasNext()) {
            String key = (String) itr.next();
            Object obj = json.get(key);
            if (obj instanceof JSONObject) {
                JSONObject innerJson = reverseIterateJSONObject((JSONObject) obj);
                res.put(key, innerJson);
            } else if (obj instanceof JSONArray) {
                JSONArray innerJson = reverseIterateJSONArray((JSONArray) obj);
                res.put(key, innerJson);
            } else {
                if (obj instanceof Long && stringMap.containsKey((Long) obj)) {
                    res.put(key, stringMap.get((Long) obj));
                } else {
                    res.put(key, obj);
                }
            }
        }
        return res;
    }

    private JSONArray reverseIterateJSONArray(JSONArray json) throws JSONException {
        JSONArray res = new JSONArray();
        for (int i = 0; i < json.length(); i++) {
            Object obj = json.get(i);
            if (obj instanceof JSONObject) {
                JSONObject innerJson = reverseIterateJSONObject((JSONObject) obj);
                res.put(i, innerJson);
            } else if (obj instanceof JSONArray) {
                JSONArray innerJson = reverseIterateJSONArray((JSONArray) obj);
                res.put(i, innerJson);
            } else {
                if (obj instanceof Long && stringMap.containsKey((Long) obj)) {
                    res.put(i, stringMap.get((Long) obj));
                } else {
                    res.put(i, obj);
                }
            }
        }
        return res;
    }

    static private synchronized Long getUID() {
        return uid++;
    }
}

